package snow;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import static snow.SWConstants.*;

public class SWControlPanel
  extends JPanel
  implements ActionListener, ChangeListener, ItemListener
{
  static final Color BACKGROUNDCOLOR = new Color(232, 232, 232);
  static final String CORNERS_STRING = "snow.SWControlPanel.CORNERS";
  static final String SIDES_STRING = "snow.SWControlPanel.SIDES";
  static final int BORDER_PADDING = 5;
  
  SWGUIModel guiModel;
  SpinnerNumberModel widthModel;
  SpinnerNumberModel heightModel;
  SpinnerNumberModel plowsModel;
  SpinnerNumberModel lengthModel;
  SpinnerListener sl = new SpinnerListener();
  JCheckBox boundaryOnlyBox;
  JCheckBox allowRetraceBox;
  JCheckBox walkThroughBox;
  JLabel scoreLabel;
  JLabel timeLabel;
  JLabel maxStreetsLabel;
  JLabel streetsLabel;
  JLabel plowsLabel;
  JLabel plowsLeftLabel;
  JLabel totalStreetsPlowedLabel;
  JLabel totalStreetsLeftLabel;
  JPanel gridCityPane;
  JPanel plowingRulesPane;
  JPanel outputPane;
  boolean sizesSet = false;
  SnowWalkerFrame parent;
  
  public SWControlPanel(SnowWalkerFrame _parent)
  {
    parent = _parent;
    guiModel = parent.guiModel;
    guiModel.addChangeListener(this);
    setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
    setBackground(BACKGROUNDCOLOR);
    setBorder(BorderFactory.createMatteBorder(1, 1, 1, 0, Color.BLACK));

    gridCityPane = makeGridCityPane();
    plowingRulesPane = makePlowingRulesPane();
    outputPane = makeOutputPane();
    shrink(gridCityPane);
    shrink(plowingRulesPane);
    addTo(this, gridCityPane);
    addTo(this, plowingRulesPane);
    addTo(this, outputPane);    
  }

  void shrink(Component c)
  {
    Dimension d = c.getMinimumSize();
    c.setPreferredSize(d);
    c.setMaximumSize(d);
  }

  void addTo(Container ct, Component c)
  {
    c.setBackground(getBackground());
    ct.add(c);
  }

  void addTo(Container ct, Component c, Object constraints)
  {
    c.setBackground(getBackground());
    ct.add(c, constraints);
  }

  Border emptyBorder()
  {
    return
    BorderFactory.createEmptyBorder(
      BORDER_PADDING,
      BORDER_PADDING,
      BORDER_PADDING,
      BORDER_PADDING);
  }

  Border titledPaddedBorder(String title)
  {
    return
      BorderFactory.createCompoundBorder(
        emptyBorder(),
        BorderFactory.createTitledBorder(title));
  }

  JPanel makeTwoSpinnerPane
  (
    SpinnerNumberModel sm1,
    String label1,
    SpinnerNumberModel sm2,
    String label2,
    boolean labelsOnLeft
  )
  {
    JPanel fieldPane = new JPanel(new GridLayout(0, 1));
    addTo(fieldPane, new JSpinner(sm1));
    addTo(fieldPane, new JSpinner(sm2));

    JPanel labelPane = new JPanel(new GridLayout(0, 1));
    addTo(labelPane, new JLabel(label1));
    addTo(labelPane, new JLabel(label2));

    JPanel retPane = new JPanel(new BorderLayout());
    addTo(retPane, fieldPane, BorderLayout.CENTER);
    String labelLayout =
      labelsOnLeft ? BorderLayout.LINE_START : BorderLayout.LINE_END;
    addTo(retPane, labelPane, labelLayout);

    return retPane;
  }

  JPanel makeDimPane()
  {
    widthModel =
      new SpinnerNumberModel(guiModel.width(), MIN_DIM, MAX_DIM, 1);
    heightModel =
      new SpinnerNumberModel(guiModel.height(), MIN_DIM, MAX_DIM, 1);
    widthModel.addChangeListener(sl);
    heightModel.addChangeListener(sl);
    return
      makeTwoSpinnerPane(
        widthModel, " blocks across", heightModel, " blocks down", false);
  }

  JPanel makeEntrancesPane()
  {
    JPanel entrancesPane = new JPanel();
    entrancesPane.setLayout(new BoxLayout(entrancesPane, BoxLayout.PAGE_AXIS));
    JLabel entrancesLabel = new JLabel("Building entrances on:");
    entrancesLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
    addTo(entrancesPane, entrancesLabel);

    JRadioButton cornerButton = new JRadioButton("corners");
    cornerButton.setModel(
      new DefaultButtonModel()
      {
        public boolean isSelected()
        {
          return guiModel.corners();
        }
      });
    cornerButton.setActionCommand(CORNERS_STRING);
    cornerButton.addActionListener(this);

    JRadioButton sideButton = new JRadioButton("sides");
    sideButton.setModel(
      new DefaultButtonModel()
      {
        public boolean isSelected()
        {
          return !guiModel.corners();
        }
      });
    sideButton.setActionCommand(SIDES_STRING);
    sideButton.addActionListener(this);

    JPanel choicesPane = new JPanel();
    addTo(choicesPane, cornerButton);
    addTo(choicesPane, sideButton);
    addTo(entrancesPane, choicesPane);
    entrancesPane.setBorder(emptyBorder());

    return entrancesPane;
  }

  JPanel makeGridCityPane()
  {
    JPanel gcp = new JPanel();
    gcp.setLayout(new BoxLayout(gcp, BoxLayout.PAGE_AXIS));
    JPanel dimPane = makeDimPane();
    dimPane.setBorder(emptyBorder());
    addTo(gcp, dimPane);
    addTo(gcp, makeEntrancesPane());
    walkThroughBox = new JCheckBox("Walk through buildings");
    walkThroughBox.setSelected(guiModel.walkThrough());
    walkThroughBox.addItemListener(this);
    walkThroughBox.setAlignmentX(Component.CENTER_ALIGNMENT);
    addTo(gcp, walkThroughBox);
    JButton configureCostButton = new JButton(parent.configureCostsAction);
    configureCostButton.setAlignmentX(Component.CENTER_ALIGNMENT);
    addTo(gcp, configureCostButton);
    gcp.setBorder(titledPaddedBorder("Grid City"));
    return gcp;
  }

  JPanel makeNumbersPane()
  {
    plowsModel =
      new SpinnerNumberModel(guiModel.maxPlows(), MIN_PLOWS, MAX_PLOWS, 1);
    lengthModel =
      new SpinnerNumberModel(guiModel.maxTotalStreets(), MIN_LENGTH, MAX_LENGTH, 1);
    plowsModel.addChangeListener(sl);
    lengthModel.addChangeListener(sl);
    return
      makeTwoSpinnerPane(
        plowsModel, "Max # plows:", lengthModel, "Max total streets:", true);
  }

  JPanel makeCheckboxPane()
  {
    boundaryOnlyBox = new JCheckBox("Only start on boundary");
    boundaryOnlyBox.setSelected(guiModel.boundaryOnly());
    boundaryOnlyBox.addItemListener(this);
    allowRetraceBox = new JCheckBox("Allow retrace");
    allowRetraceBox.setSelected(guiModel.allowRetrace());
    allowRetraceBox.addItemListener(this);
    JPanel checkPane = new JPanel(new GridLayout(0, 1));
    addTo(checkPane, boundaryOnlyBox);
    addTo(checkPane, allowRetraceBox);
    return checkPane;
  }

  JPanel makePlowingRulesPane()
  {
    JPanel prp = new JPanel();
    prp.setLayout(new BoxLayout(prp, BoxLayout.PAGE_AXIS));

    JPanel outerNumbersPane = new JPanel();
    outerNumbersPane.setLayout(
      new BoxLayout(outerNumbersPane, BoxLayout.PAGE_AXIS));
    addTo(outerNumbersPane, makeNumbersPane());
    JLabel commentLabel = new JLabel("(0 = no limit)");
    commentLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
    addTo(outerNumbersPane, commentLabel);
    outerNumbersPane.setBorder(emptyBorder());
    addTo(prp, outerNumbersPane);
    addTo(prp, makeCheckboxPane());
    prp.setBorder(titledPaddedBorder("Plowing Rules"));
    return prp;
  }

  JPanel makeOutputPane()
  {
    JPanel outputPane = new JPanel();
    outputPane.setLayout(new BorderLayout());
    JPanel innerPane = new JPanel();
    innerPane.setLayout(new GridLayout(0, 1));
    scoreLabel = new JLabel();
    timeLabel = new JLabel();
    maxStreetsLabel = new JLabel();
    streetsLabel = new JLabel();
    plowsLabel = new JLabel();
    plowsLeftLabel = new JLabel();
    totalStreetsPlowedLabel = new JLabel();
    totalStreetsLeftLabel = new JLabel();
    updateLabels();
    addTo(innerPane, scoreLabel);
    addTo(innerPane, timeLabel);
    addTo(innerPane, maxStreetsLabel);
    addTo(innerPane, streetsLabel);
    addTo(innerPane, plowsLabel);
    addTo(innerPane, plowsLeftLabel);
    addTo(innerPane, totalStreetsPlowedLabel);
    addTo(innerPane, totalStreetsLeftLabel);
    addTo(outputPane, innerPane, BorderLayout.NORTH);
    outputPane.setBorder(titledPaddedBorder("Output"));
    JPanel buttonPane = new JPanel();
    buttonPane.setLayout(new GridLayout(0,1));
    JButton undoButton = new JButton(guiModel.undoAction);
    JButton resetButton = new JButton(guiModel.resetAction);
    addTo(buttonPane, undoButton);
    addTo(buttonPane, resetButton);
    addTo(outputPane, buttonPane, BorderLayout.SOUTH);
    return outputPane;
  }

  public void actionPerformed(ActionEvent e)
  {
    String cmd = e.getActionCommand();
    if (CORNERS_STRING.equals(cmd))
    {
      guiModel.corners(true);
    }
    else if (SIDES_STRING.equals(cmd))
    {
      guiModel.corners(false);
    }
  }

  void updateLabels()
  {
    scoreLabel.setText("Score: " + SWGUIModel.formatDouble(guiModel.score()));
    int minutes = guiModel.maxStreets() * PLOW_MINUTES;
    timeLabel.setText("Time: " + minutes + " minutes");
    maxStreetsLabel.setText("Max # Streets: " + guiModel.maxStreets());
    streetsLabel.setText("# Streets: " + guiModel.streets());
    plowsLabel.setText("Plows Used: " + guiModel.plowsUsed());
    plowsLeftLabel.setText(
      "Plows Left: " + SWGUIModel.formatInt(guiModel.plowsRemaining()));
    totalStreetsPlowedLabel.setText("Total Streets Plowed: " + guiModel.totalStreetsPlowed());
    totalStreetsLeftLabel.setText(
      "Total Streets Left: " + SWGUIModel.formatInt(guiModel.totalStreetsRemaining()));
  }

  void updateControls()
  {
    widthModel.setValue(guiModel.width());
    heightModel.setValue(guiModel.height());
    plowsModel.setValue(guiModel.maxPlows());
    lengthModel.setValue(guiModel.maxTotalStreets());
    boundaryOnlyBox.setSelected(guiModel.boundaryOnly());
    allowRetraceBox.setSelected(guiModel.allowRetrace());
    walkThroughBox.setSelected(guiModel.walkThrough());
  }
  
  public void stateChanged(ChangeEvent e)
  {
    updateLabels();
    updateControls();
    repaint();
  }

  public void itemStateChanged(ItemEvent e)
  {
    Object source = e.getItemSelectable();
    boolean b = e.getStateChange() == ItemEvent.SELECTED;
    if (source == boundaryOnlyBox)
    {
      guiModel.setBoundaryOnly(b);
    }
    else if (source == allowRetraceBox)
    {
      guiModel.setAllowRetrace(b);
    }
    else if (source == walkThroughBox)
    {
      guiModel.setWalkThrough(b);
    }
  }

  class SpinnerListener implements ChangeListener
  {
    public void stateChanged(ChangeEvent e)
    {
      SpinnerNumberModel snm = (SpinnerNumberModel)e.getSource();
      int newVal = snm.getNumber().intValue();
      if (snm == widthModel)
      {
        guiModel.setDimension(newVal, guiModel.height());
      }
      else if (snm == heightModel)
      {
        guiModel.setDimension(guiModel.width(), newVal);
      }
      else if (snm == plowsModel)
      {
        guiModel.setMaxPlows(newVal);
      }
      else if (snm == lengthModel)
      {
        guiModel.setMaxTotalStreets(newVal);
      }
    }
  }
}

