package snow;

import java.awt.event.*;
import java.beans.*;
import javax.swing.*;

import static snow.SWConstants.*;

public class SWScriptDialog
  extends JDialog implements ActionListener, PropertyChangeListener
{
  SWGUIModel guiModel;
  JOptionPane optionPane;
  JTextArea textArea;
  String saveButton = "Run";
  String cancelButton = "Cancel";
  String currentToken;
  
  SWScriptDialog(SWGUIModel _guiModel, JFrame parent)
  {
    super(parent, true);
    setTitle("Run Script");
    guiModel = _guiModel;
    textArea = new JTextArea(8, SCRIPT_WORD_WRAP + 4);
    textArea.setLineWrap(true);
    JScrollPane scrollPane = new JScrollPane(textArea);
    JLabel spacer = new JLabel("     ");
    Object[] array =
      {
        "Specify a sequence of actions to run.",
        "Actions are case-insensitive.",
        "Whitespace is ignored.",
        spacer,
        "You can specify:",
        "    N,E,S,W to move in the corresponding direction",
        "    " + START_PLOW_STRING + " to start plowing",
        "    " + END_PLOW_STRING + " to end plowing",
        scrollPane,
      };
    Object[] options = { saveButton, cancelButton };
    optionPane =
      new JOptionPane(
        array,
        JOptionPane.PLAIN_MESSAGE,
        JOptionPane.OK_CANCEL_OPTION,
        null,
        options,
        saveButton);
    setContentPane(optionPane);
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(
      new WindowAdapter()
      {
        public void windowClosing(WindowEvent we)
        {
          /*
           * Instead of directly closing the window,
           * we're going to change the JOptionPane's
           * value property.
           */
          optionPane.setValue(new Integer(JOptionPane.CLOSED_OPTION));
        }
      });
    optionPane.addPropertyChangeListener(this);
    addComponentListener(
      new ComponentAdapter()
      {
        public void componentShown(ComponentEvent ce)
        {
          textArea.requestFocusInWindow();
        }
      });
    pack();
  }

  public void setVisible(boolean b)
  {
    if (b)
    {
      textArea.setText(guiModel.script());
      textArea.selectAll();
    }
    super.setVisible(b);
  }

  public void actionPerformed(ActionEvent e)
  {
    optionPane.setValue(saveButton);
  }

  public void propertyChange(PropertyChangeEvent e)
  {
    String prop = e.getPropertyName();

    if (isVisible() &&
         (e.getSource() == optionPane) &&
         (JOptionPane.VALUE_PROPERTY.equals(prop) ||
           JOptionPane.INPUT_VALUE_PROPERTY.equals(prop)))
    {
      Object value = optionPane.getValue();
      if (value == JOptionPane.UNINITIALIZED_VALUE)
      {
        //ignore reset
        return;
      }

      //Reset the JOptionPane's value.
      //If you don't do this, then if the user
      //presses the same button next time, no
      //property change event will be fired.
      optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);

      if (saveButton.equals(value))
      {
        String script = textArea.getText();
        String error = guiModel.processScript(script);
        if (error != null)
        {
          // don't close window
          textArea.requestFocusInWindow();
          return;
        }
      }
      // else cancel or close window - do nothing
        
      setVisible(false);
    }
  }

}
