package snow;

import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import java.util.*;
import static snow.SWConstants.*;

public class SWSetPuzzleDialog
  extends JDialog implements ActionListener, PropertyChangeListener
{
  SWGUIModel guiModel;
  JOptionPane optionPane;
  JTextArea textArea;
  String saveButton = "Set";
  String defaultsButton = "Defaults";
  String cancelButton = "Cancel";
  String currentToken;
  
  SWSetPuzzleDialog(SWGUIModel _guiModel, JFrame parent)
  {
    super(parent, true);
    setTitle("Set Puzzle");
    guiModel = _guiModel;
    textArea = new JTextArea(9, 20);
    JScrollPane scrollPane = new JScrollPane(textArea);
    JLabel spacer = new JLabel("     ");
    Object[] array =
      {
        "Specify the puzzle parameters.",
        "Use spaces, tabs, or new lines to separate items.",
        "Parameters are case-insensitive.",
        "Parameters that are not specified will be left unchanged.",
        spacer,
        "You can specify:",
        "    grid <width> <height>",
        "    entrances [corners|sides]",
        "    walkthrough [yes|no]",
        "    costs <adj corner> <opp corner> <adj side> <opp side>",
        "    maxPlows <n>",
        "    maxTotalStreets <n>",
        "    onlyStartBoundary [yes|no]",
        "    allowRetrace [yes|no]",
        scrollPane,
      };
    Object[] options = { saveButton, defaultsButton, cancelButton };
    optionPane =
      new JOptionPane(
        array,
        JOptionPane.PLAIN_MESSAGE,
        JOptionPane.YES_NO_CANCEL_OPTION,
        null,
        options,
        saveButton);
    setContentPane(optionPane);
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(
      new WindowAdapter()
      {
        public void windowClosing(WindowEvent we)
        {
          /*
           * Instead of directly closing the window,
           * we're going to change the JOptionPane's
           * value property.
           */
          optionPane.setValue(new Integer(JOptionPane.CLOSED_OPTION));
        }
      });
    optionPane.addPropertyChangeListener(this);
    addComponentListener(
      new ComponentAdapter()
      {
        public void componentShown(ComponentEvent ce)
        {
          textArea.requestFocusInWindow();
        }
      });
    pack();
  }

  public void setVisible(boolean b)
  {
    if (b)
    {
      java.util.List<String> ss = new ArrayList<String>();
      ss.add(
        String.format("grid %d %d", guiModel.width(), guiModel.height()));
      String cs = guiModel.corners() ? "corners" : "sides";
      ss.add("entrances " + cs);
      String wt = SWGUIModel.formatBoolean(guiModel.walkThrough());
      ss.add("walkthrough " + wt.toLowerCase());
      if (guiModel.walkThrough())
      {
        StringBuilder costs = new StringBuilder();
        for (int i = 0; i < COSTS; ++i)
        {
          costs.append(" " + guiModel.walkThroughCost(i)); 
        }
        ss.add("costs" + costs.toString());
      }
      ss.add("maxPlows " + guiModel.maxPlows());
      ss.add("maxTotalStreets " + guiModel.maxTotalStreets());
      String osb = SWGUIModel.formatBoolean(guiModel.boundaryOnly());
      ss.add("onlyStartBoundary " + osb.toLowerCase());
      String ar = SWGUIModel.formatBoolean(guiModel.allowRetrace());
      ss.add("allowRetrace " + ar.toLowerCase());
      StringBuilder sb = new StringBuilder();
      for (String s : ss)
      {
        sb.append(s);
        sb.append("\n");
      }
      textArea.setText(sb.toString());
      textArea.selectAll();
    }
    super.setVisible(b);
  }

  public void actionPerformed(ActionEvent e)
  {
    optionPane.setValue(saveButton);
  }

  public void propertyChange(PropertyChangeEvent e)
  {
    String prop = e.getPropertyName();

    if (isVisible() &&
         (e.getSource() == optionPane) &&
         (JOptionPane.VALUE_PROPERTY.equals(prop) ||
           JOptionPane.INPUT_VALUE_PROPERTY.equals(prop)))
    {
      Object value = optionPane.getValue();
      if (value == JOptionPane.UNINITIALIZED_VALUE)
      {
        //ignore reset
        return;
      }

      //Reset the JOptionPane's value.
      //If you don't do this, then if the user
      //presses the same button next time, no
      //property change event will be fired.
      optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);

      if (saveButton.equals(value))
      {
        String[] rawtokens = textArea.getText().split("\\s");
        LinkedList<String> previewTokens = new LinkedList<String>();
        for (String s : rawtokens)
        {
          if (!s.equals(""))
          {
            previewTokens.add(s);
          }
        }
        LinkedList<String> realTokens = new LinkedList<String>();
        realTokens.addAll(previewTokens);
        String error = null;
        try
        {
          processTokens(previewTokens, false);
          guiModel.startBatch();
          processTokens(realTokens, true);
        }
        catch (NoSuchElementException nsex)
        {
          error = "Not enough arguments to \"" + currentToken + "\"";
        }
        catch (SWParseException swpx)
        {
          error = swpx.getMessage();
        }
        finally
        {
          guiModel.endBatch();
        }
        if (error != null)
        {
          JOptionPane.showMessageDialog(
            null, error, "Error", JOptionPane.ERROR_MESSAGE);
          textArea.requestFocusInWindow();
          return;          
        }
      }
      else if (defaultsButton.equals(value))
      {
        guiModel.restoreDefaults();
      }
      // else cancel or close window - do nothing
      
      setVisible(false);
    }
  }
  
  void processTokens(LinkedList<String> tokens, boolean doIt)
    throws SWParseException
  {
    while (!tokens.isEmpty())
    {
      currentToken = tokens.removeFirst();
      if ("grid".equalsIgnoreCase(currentToken))
      {
        int w = getValidatedInt(tokens.removeFirst(), MIN_DIM, MAX_DIM);
        int h = getValidatedInt(tokens.removeFirst(), MIN_DIM, MAX_DIM);
        if (doIt)
        {
          guiModel.setDimension(w, h);
        }
      }
      else if ("entrances".equalsIgnoreCase(currentToken))
      {
        boolean corners =
          getValidatedBinary(tokens.removeFirst(), "corners", "sides");
        if (doIt)
        {
          guiModel.corners(corners);
        }
      }
      else if ("walkthrough".equalsIgnoreCase(currentToken))
      {
        boolean b = getValidatedYesNo(tokens.removeFirst());
        if (doIt)
        {
          guiModel.setWalkThrough(b);
        }
      }
      else if ("costs".equalsIgnoreCase(currentToken))
      {
        double costs[] = new double[COSTS];
        for (int i = 0; i < COSTS; ++i)
        {
          costs[i] = getNonnegativeCost(tokens.removeFirst());
        }
        if (doIt)
        {
          guiModel.setWalkThroughCosts(costs);
        }
      }
      else if ("maxPlows".equalsIgnoreCase(currentToken))
      {
        int n = getValidatedInt(tokens.removeFirst(), MIN_PLOWS, MAX_PLOWS);
        if (doIt)
        {
          guiModel.setMaxPlows(n);
        }
      }
      else if ("maxTotalStreets".equalsIgnoreCase(currentToken))
      {
        int n = getValidatedInt(tokens.removeFirst(), MIN_LENGTH, MAX_LENGTH);
        if (doIt)
        {
          guiModel.setMaxTotalStreets(n);
        }
      }
      else if ("onlyStartBoundary".equalsIgnoreCase(currentToken))
      {
        boolean b = getValidatedYesNo(tokens.removeFirst());
        if (doIt)
        {
          guiModel.setBoundaryOnly(b);
        }
      }
      else if ("allowRetrace".equalsIgnoreCase(currentToken))
      {
        boolean b = getValidatedYesNo(tokens.removeFirst());
        if (doIt)
        {
          guiModel.setAllowRetrace(b);
        }
      }
      else
      {
        throw
          new
            SWParseException("Unknown parameter \"" + currentToken + "\".");
      }
    }
  }
  
  int getValidatedInt(String token, int min, int max) throws SWParseException
  {
    int dim = Integer.parseInt(token);
    if (dim < min)
    {
      String error =
        String.format(
          "Argument to \"%s\" must be at least %d (received %d).",
          currentToken,
          min,
          dim);
      throw new SWParseException(error);
    }
    if (dim > max)
    {
      String error =
        String.format(
          "Argument to \"%s\" can be at most %d (received %d).",
          currentToken,
          max,
          dim);
      throw new SWParseException(error);
    }
    return dim;
  }
  
  boolean getValidatedBinary
  (
    String token,
    String trueString,
    String falseString
  ) throws SWParseException
  {
    if (trueString.equalsIgnoreCase(token))
    {
      return true;
    }
    if (falseString.equalsIgnoreCase(token))
    {
      return false;
    }
    String error =
      String.format(
        "Argument to \"%s\" must be " +
        "\"%s\" or \"%s\" (received \"%s\").",
        currentToken,
        trueString,
        falseString,
        token);
    throw new SWParseException(error);
  }
  
  boolean getValidatedYesNo(String token) throws SWParseException
  {
    return getValidatedBinary(token, "yes", "no");
  }
  
  double getNonnegativeCost(String token) throws SWParseException
  {
    double d = Double.parseDouble(token);
    if (d < 0)
    {
      String error =
        String.format(
          "Argument to \"%s\" must be nonnegative (received %.1f).",
          currentToken,
          d);
      throw new SWParseException(error);
    }
    return d;
  }
}
